@@ -89,6 +89,9 @@
 #' @param output_level When \code{output_level = 1}, return basic cos details for colocalization results
 #'                     When \code{output_level = 2}, return the ucos details for the single specific effects.
 #'                     When \code{output_level = 3}, return the entire Colocboost model to diagnostic results (more space).
+#' @param cos_npc_cutoff Minimum threshold of normalized probability of colocalization (NPC) for CoS.
+#' @param npc_outcome_cutoff Minimum threshold of normalized probability of colocalized traits in each CoS.
+#' @param pvalue_cutoff Maximum threshold of marginal p-values of colocalized variants on colocalized traits in each CoS.
 #'
 #' @return A \code{"colocboost"} object with some or all of the following elements:
 #'
@@ -184,7 +187,12 @@ colocboost <- function(X = NULL, Y = NULL, # individual data
                        check_null_max_ucos = 0.015, # the smallest value of change of profile loglikelihood for each outcome in uCoS.
                        weaker_effect = TRUE,
                        LD_free = FALSE,
-                       output_level = 1) {
+                       output_level = 1,
+                       ###### - Post filtering parameters
+                       cos_npc_cutoff = 0.2, # remove the CoS with cos_npc less than this cutoff
+                       npc_outcome_cutoff = 0.2, # remove the colocalized outcome in CoS if npc_outcome less than this cutoff
+                       pvalue_cutoff = 1e-3 # remove the colocalized outcome in CoS if pvalue greater than this cutoff
+                       ) {
   ###################### ---- one module for data object
   message("Validating input data.")
   # - check if all missing
@@ -195,344 +203,39 @@ colocboost <- function(X = NULL, Y = NULL, # individual data
     return(NULL)
   }
 
-  # - check individual level data
-  if (!is.null(X) & !is.null(Y)) {
-    # --- check input
-    if (is.data.frame(X))  X <- as.matrix(X)
-    if (is.data.frame(Y)) Y <- as.matrix(Y)
-    if (is.matrix(X))  X <- list(X)
-    if (is.atomic(Y) && !is.list(Y)) {
-      Y <- as.matrix(Y)
-      if (ncol(Y) == 1) {
-        Y <- list(Y)
-      } else {
-        n <- nrow(X[[1]])
-        if (nrow(Y) == n) {
-          Y <- lapply(1:ncol(Y), function(i) Y[, i, drop = FALSE])
-        } else if (ncol(Y) == n) {
-          Y <- lapply(1:nrow(Y), function(i) t(Y[i, , drop = FALSE]))
-        } else {
-          stop("X and Y do not have the same sample size!")
-        }
-      }
-    } else {
-      Y <- lapply(1:length(Y), function(ii) {
-        y <- Y[[ii]]
-        y <- as.matrix(y)
-        n <- length(y)
-        if (nrow(y) == n) {
-          return(y)
-        } else if (ncol(y) == n) {
-          return(t(y))
-        } 
-      })
-    }
-    
-    # --- check if variables in individual data
-    p.ind <- unique(sapply(X, ncol))
-    if (length(p.ind) != 1) {
-      variable.tmp <- sapply(X, function(xx) {
-        if (!is.null(colnames(xx))) TRUE else FALSE
-      })
-      if (!all(variable.tmp)) {
-        warning("Error: X matrices do not have the same number of variables. Provide variable names to the colnames of X matrix.")
-        return(NULL)
-      }
-    } else {
-      # --- check if there is only one X, default variable names as X_1, ..., X_p
-      X <- lapply(X, function(xx) {
-        if (is.null(colnames(xx))) {
-          colnames(xx) <- paste0("X_", 1:ncol(xx))
-        }
-        return(xx)
-      })
-    }
-
-    keep_variable_individual <- lapply(X, colnames)
-    if (!is.list(X) & !is.list(Y)) {
-      warning("Error: Input X and Y must be the list containing genotype matrics and all phenotype vectors!")
-      return(NULL)
-    } else {
-      if (length(X) == 1) {
-        yx_dict <- rep(1, length(Y))
-      } else if (length(X) == length(Y)) {
-        yx_dict <- 1:length(Y)
-      } else {
-        if (is.null(dict_YX)) {
-          warning("Error: Please provide the dict_YX since you have multiple Y but only few X!")
-          return(NULL)
-        } else {
-          # - dict for Y to X mapping
-          yx_dict <- rep(NA, length(Y))
-          for (i in 1:length(Y)) {
-            tmp <- unique(dict_YX[dict_YX[, 1] == i, 2])
-            if (length(tmp) == 0) {
-              warning(paste("Error: You don't provide matched X for outcome", i))
-              return(NULL)
-            } else if (length(tmp) != 1) {
-              warning(paste("Error: You provide different matched X for outcome", i))
-              return(NULL)
-            } else {
-              yx_dict[i] <- tmp
-            }
-          }
-          # - check if Y and X all mapped
-          if (max(yx_dict) > length(X)) {
-            warning("Error: You don't provide enough X matrices!")
-            return(NULL)
-          }
-        }
-      }
-    }
-    # keep_variable_individual <- lapply(yx_dict, function(i) keep_variable_individual[[i]] )
-    if (any(sapply(X, anyNA))) {
-      warning("Error: Input X must not contain missing values.")
-      return(NULL)
-    }
-    if (any(sapply(Y, anyNA))) {
-      pos <- which(sapply(Y, anyNA))
-      if (length(pos) == 1) {
-        Y_miss <- Y[[pos]]
-        samples_kept <- which(!is.na(Y_miss))
-        Y[[pos]] <- as.matrix(Y_miss[samples_kept])
-        # - check if only y for x
-        x_dict <- yx_dict[pos]
-        X_miss <- X[[x_dict]]
-        X_miss <- X_miss[samples_kept, ]
-        if (sum(yx_dict == x_dict) != 1) {
-          x_dict <- max(yx_dict) + 1
-          yx_dict[x_dict] <- x_dict
-        }
-        X[[x_dict]] <- X_miss
-      } else {
-        for (i in pos) {
-          Y_miss <- Y[[i]]
-          samples_kept <- which(!is.na(Y_miss))
-          Y[[i]] <- as.matrix(Y_miss[samples_kept])
-          # - check if only y for x
-          x_dict <- yx_dict[i]
-          X_miss <- X[[x_dict]]
-          X_miss <- X_miss[samples_kept, ]
-          if (sum(yx_dict == x_dict) != 1) {
-            x_dict <- max(yx_dict) + 1
-            yx_dict[x_dict] <- x_dict
-          }
-          X[[x_dict]] <- X_miss
-        }
-      }
-    }
-  } else {
-    yx_dict <- keep_variable_individual <- NULL
-  }
-
-  # - check summary-level data
-  if ((!is.null(sumstat)) | (!is.null(effect_est) & !is.null(effect_se))) {
-    # --- check input of (effect_est, effect_se)
-    if ((!is.null(effect_est) & !is.null(effect_se))) {
-      if (!all(dim(effect_est) == dim(effect_se))) {
-        warning("Error: effect_est and effect_se should be the same dimension! Please check!")
-        return(NULL)
-      }
-      variables <- rownames(effect_est)
-      effect_est <- as.matrix(effect_est)
-      effect_se <- as.matrix(effect_se)
-      if (is.null(variables)) {
-        variables <- paste0("variant_", 1:nrow(effect_est))
-      }
-      sumstat <- list()
-      for (sum_iy in 1:ncol(effect_est)) {
-        sumstat_y <- data.frame(
-          "beta" = effect_est[, sum_iy],
-          "sebeta" = effect_se[, sum_iy],
-          "variant" = variables
-        )
-        if (!is.null(effect_n)) {
-          if (length(effect_n) == 1) {
-            sumstat_y$n <- effect_n
-          } else if (length(effect_n) == ncol(effect_est)) {
-            sumstat_y$n <- effect_n[sum_iy]
-          } else {
-            warning("effect_n does not match effect_est, ignoring sample size!")
-          }
-        }
-        sumstat[[sum_iy]] <- sumstat_y
-      }
-    }
-
-    if (!is.null(sumstat)) {
-      if (is.data.frame(sumstat)) {
-        sumstat <- list(sumstat)
-      }
-      if (!is.list(sumstat)) {
-        warning("Error: Input sumstat must be the list containing summary level data for all outcomes!")
-        return(NULL)
-      }
-      # --- check if variables names in summary data
-      variable.tmp <- sapply(sumstat, function(xx) {
-        if (("variant" %in% colnames(xx))) {
-          TRUE
-        } else {
-          FALSE
-        }
-      })
-      if (!all(variable.tmp)) {
-        warning(
-          "sumstat do not contain variables names. Assigning default names as 'variant_1, ..., variant_P'. ",
-          "Recommend to provide variables names to the column named 'variant'."
-        )
-        sumstat <- lapply(sumstat, function(xx) {
-          if (!("variant" %in% colnames(xx))) {
-            xx$variant <- paste0("variant_", 1:nrow(xx))
-          }
-          return(xx)
-        })
-      }
-    }
-    keep_variable_sumstat <- lapply(sumstat, function(xx) {
-      xx$variant
-    })
-
-    # --- check input of LD
-    if (is.null(LD)) {
-      # if no LD input, set diagonal matrix to LD
-      warning(
-        "Providing the LD for summary statistics data is highly recommended. ",
-        "Without LD, only a single iteration will be performed under the assumption of one causal variable per outcome. ",
-        "Additionally, the purity of CoS cannot be evaluated!"
-      )
-      
-      LD <- 1
-      sumstatLD_dict <- rep(1, length(sumstat))
-      # change some algorithm parameters
-      M <- 1 # one iteration
-      min_abs_corr <- 0 # remove purity checking
-      jk_equiv_corr <- 0
-      jk_equiv_loglik <- 0.1
-      func_simplex <- "only_z2z"
-    } else {
-      if (is.data.frame(LD)) {
-        LD <- as.matrix(LD)
-      }
-      if (is.matrix(LD)) {
-        LD <- list(LD)
-      }
-      # - check if NA in LD matrix
-      num_na <- sapply(LD, sum)
-      if (any(is.na(num_na))){
-        warning("Error: Input LD must not contain missing values (NA).")
-        return(NULL)
-      }
-      if (length(LD) == 1) {
-        sumstatLD_dict <- rep(1, length(sumstat))
-      } else if (length(LD) == length(sumstat)) {
-        sumstatLD_dict <- 1:length(sumstat)
-      } else {
-        if (is.null(dict_sumstatLD)) {
-          warning("Error: Please provide the dict_sumstatLD since you have multiple sumstat but only few LD!")
-          return(NULL)
-        } else {
-          # - dict for sumstat to LD mapping
-          sumstatLD_dict <- rep(NA, length(sumstat))
-          for (i in 1:length(sumstat)) {
-            tmp <- unique(dict_sumstatLD[dict_sumstatLD[, 1] == i, 2])
-            if (length(tmp) == 0) {
-              warning(paste("Error: You don't provide matched LD for sumstat", i))
-              return(NULL)
-            } else if (length(tmp) != 1) {
-              warning(paste("Error: You provide multiple matched LD for sumstat", i))
-              return(NULL)
-            } else {
-              sumstatLD_dict[i] <- tmp
-            }
-          }
-          if (max(sumstatLD_dict) > length(LD)) {
-            warning("Error: You don't provide enough LD matrices!")
-            return(NULL)
-          }
-        }
-      }
-    }
-
-    # - checking sample size existency
-    n_exist <- sapply(sumstat, function(ss) {
-      "n" %in% colnames(ss)
-    })
-    if (!all(n_exist)) {
-      # The sample size (n) is not provided, so use unadjusted z-scores.
-      # The choice of n=2, yty=1 is mostly arbitrary except in that it
-      # ensures var(y) = yty/(n-1) = 1
-      p_no <- which(!n_exist)
-      warning(
-        "Providing the sample size (n), or even a rough estimate of n, ",
-        "is highly recommended. Without n, the implicit assumption is ",
-        "n is large (Inf) and the effect sizes are small (close to zero). ",
-        "Outcome ", paste(p_no, collapse = ", "), " in sumstat don't contain 'n'!"
-      )
-    }
-
-    Z <- N_sumstat <- Var_y <- SeBhat <- vector(mode = "list", length = length(sumstat))
-    for (i.summstat in 1:length(sumstat)) {
-      summstat_tmp <- sumstat[[i.summstat]]
-      colVar <- colnames(summstat_tmp)
-      # - check p-value
-      if (!("z" %in% colVar)) {
-        if (!("beta" %in% colVar) || !("sebeta" %in% colVar)) {
-          warning("Error: Please provide either z or (bhat, sebhat)!")
-          return(NULL)
-        }
-        bhat <- summstat_tmp[, "beta"]
-        sebhat <- summstat_tmp[, "sebeta"]
-        if (anyNA(bhat) || anyNA(sebhat)) {
-          warning("Error: bhat, sebhat cannot have missing values")
-          return(NULL)
-        }
-        if (any(sebhat <= 0)) {
-          warning("Error: sebhat cannot have zero or negative elements")
-          return(NULL)
-        }
-        z <- bhat / sebhat
-        SeBhat[[i.summstat]] <- sebhat
-      } else {
-        z <- summstat_tmp[, "z"]
-      }
-      if (anyNA(z)) {
-        warning(paste("summary statistic dataset", i.summstat, "contains NA values that are replaced with 0"))
-        z[is.na(z)] <- 0
-      }
-
-      # - check N
-      if (!("n" %in% colVar)) {
-        z <- z
-      } else {
-        n <- summstat_tmp[, "n"]
-        n <- as.numeric(na.omit(n)) # add hoc
-        if (!all(n > 1)) {
-          warning("Error: Sample size N must be greater than 1!")
-          return(NULL)
-        }
-        n <- median(n)
-        N_sumstat[[i.summstat]] <- n
-
-        # When n is provided, compute the adjusted z-scores.
-        z <- z * sqrt((n - 1) / (z^2 + n - 2))
-        if ("var_y" %in% colVar) {
-          var_y <- unique(summstat_tmp[, "var_y"])
-          # --- need to check
-          if (length(var_y) != 1) {
-            var_y <- 1
-          }
-          if (var_y <= 0) {
-            warning("Error: var_y cannot have zero or negative elements")
-            return(NULL)
-          }
-          Var_y[[i.summstat]] <- var_y
-        }
-      }
-      Z[[i.summstat]] <- z
-    }
-  } else {
-    Z <- N_sumstat <- Var_y <- SeBhat <- sumstatLD_dict <- keep_variable_sumstat <- NULL
+  # - check input data: individual level data and summary-level data
+  validated_data <- colocboost_validate_input_data(
+    X = X, Y = Y,
+    sumstat = sumstat, LD = LD,
+    dict_YX = dict_YX, dict_sumstatLD = dict_sumstatLD,
+    effect_est = effect_est, effect_se = effect_se, effect_n = effect_n,
+    overlap_variables = overlap_variables,
+    M = M, min_abs_corr = min_abs_corr
+  )
+  if (is.null(validated_data)) {
+    return(NULL)
   }
+  # Extract validated data
+  X <- validated_data$X
+  Y <- validated_data$Y
+  yx_dict <- validated_data$yx_dict
+  keep_variable_individual <- validated_data$keep_variable_individual
+  sumstat <- validated_data$sumstat
+  LD <- validated_data$LD
+  sumstatLD_dict <- validated_data$sumstatLD_dict
+  keep_variable_sumstat <- validated_data$keep_variable_sumstat
+  Z <- validated_data$Z
+  N_sumstat <- validated_data$N_sumstat
+  Var_y <- validated_data$Var_y
+  SeBhat <- validated_data$SeBhat
+  
+  # Update parameters if LD was not provided
+  M <- validated_data$M
+  min_abs_corr <- validated_data$min_abs_corr
+  jk_equiv_corr <- validated_data$jk_equiv_corr
+  jk_equiv_loglik <- validated_data$jk_equiv_loglik
+  func_simplex <- validated_data$func_simplex
+  
   # - initial colocboost object
   keep_variables <- c(keep_variable_individual, keep_variable_sumstat)
   overlapped_variables <- Reduce("intersect", keep_variables)
@@ -625,6 +328,505 @@ colocboost <- function(X = NULL, Y = NULL, # individual data
     output_level = output_level
   )
   class(cb_output) <- "colocboost"
+
+  # ---- post filtering of the colocboost results (get robust colocalization events)
+  cb_output <- get_robust_colocalization(
+    cb_output = cb_output,
+    cos_npc_cutoff = cos_npc_cutoff,
+    npc_outcome_cutoff = npc_outcome_cutoff,
+    pvalue_cutoff = pvalue_cutoff,
+    weight_fudge_factor = weight_fudge_factor,
+    coverage = coverage
+  )
+
   return(cb_output)
 }
 
+
+
+
+#' @title Validate and Process All Input Data for ColocBoost
+#'
+#' @description Internal function to validate and process both individual-level and summary-level input data
+#'
+#' @param X A list of genotype matrices for different outcomes, or a single matrix if all outcomes share the same genotypes.
+#' @param Y A list of vectors of outcomes or an N by L matrix if it is considered for the same X and multiple outcomes.
+#' @param sumstat A list of data.frames of summary statistics.
+#' @param LD A list of correlation matrices indicating the LD matrix for each genotype.
+#' @param dict_YX A L by 2 matrix of dictionary for X and Y if there exist subsets of outcomes corresponding to the same X matrix.
+#' @param dict_sumstatLD A L by 2 matrix of dictionary for sumstat and LD if there exist subsets of outcomes corresponding to the same sumstat.
+#' @param effect_est Matrix of variable regression coefficients (i.e. regression beta values) in the genomic region
+#' @param effect_se Matrix of standard errors associated with the beta values
+#' @param effect_n A scalar or a vector of sample sizes for estimating regression coefficients.
+#' @param overlap_variables If overlap_variables = TRUE, only perform colocalization in the overlapped region.
+#' @param M The maximum number of gradient boosting rounds for each outcome (default is 500).
+#' @param min_abs_corr Minimum absolute correlation allowed in a confidence set.
+#'
+#' @return A list containing:
+#'   \item{X}{Processed list of genotype matrices}
+#'   \item{Y}{Processed list of phenotype vectors}
+#'   \item{yx_dict}{Dictionary mapping Y to X}
+#'   \item{keep_variable_individual}{List of variable names for each X matrix}
+#'   \item{sumstat}{Processed list of summary statistics data.frames}
+#'   \item{LD}{Processed list of LD matrices}
+#'   \item{sumstatLD_dict}{Dictionary mapping sumstat to LD}
+#'   \item{keep_variable_sumstat}{List of variant names for each sumstat}
+#'   \item{Z}{List of z-scores for each outcome}
+#'   \item{N_sumstat}{List of sample sizes for each outcome}
+#'   \item{Var_y}{List of phenotype variances for each outcome}
+#'   \item{SeBhat}{List of standard errors for each outcome}
+#'   \item{M_updated}{Updated M value (may be changed if LD not provided)}
+#'   \item{min_abs_corr_updated}{Updated min_abs_corr value (may be changed if LD not provided)}
+#'   \item{jk_equiv_corr_updated}{Updated jk_equiv_corr value}
+#'   \item{jk_equiv_loglik_updated}{Updated jk_equiv_loglik value}
+#'   \item{func_simplex_updated}{Updated func_simplex value}
+#'
+#' @keywords internal
+colocboost_validate_input_data <- function(X = NULL, Y = NULL,
+                                           sumstat = NULL, LD = NULL,
+                                           dict_YX = NULL, dict_sumstatLD = NULL,
+                                           effect_est = NULL, effect_se = NULL, effect_n = NULL,
+                                           overlap_variables = FALSE,
+                                           M = 500, min_abs_corr = 0.5) {
+  
+  # - check individual level data
+  if (!is.null(X) & !is.null(Y)) {
+    # --- check input
+    if (is.data.frame(X))  X <- as.matrix(X)
+    if (is.data.frame(Y)) Y <- as.matrix(Y)
+    if (is.matrix(X))  X <- list(X)
+    if (is.atomic(Y) && !is.list(Y)) {
+      Y <- as.matrix(Y)
+      if (ncol(Y) == 1) {
+        Y <- list(Y)
+      } else {
+        n <- nrow(X[[1]])
+        if (nrow(Y) == n) {
+          Y <- lapply(1:ncol(Y), function(i) Y[, i, drop = FALSE])
+        } else if (ncol(Y) == n) {
+          Y <- lapply(1:nrow(Y), function(i) t(Y[i, , drop = FALSE]))
+        } else {
+          stop("X and Y do not have the same sample size!")
+        }
+      }
+    } else {
+      Y <- lapply(1:length(Y), function(ii) {
+        y <- Y[[ii]]
+        y <- as.matrix(y)
+        n <- length(y)
+        if (nrow(y) == n) {
+          return(y)
+        } else if (ncol(y) == n) {
+          return(t(y))
+        } 
+      })
+    }
+    
+    # --- check if variables in individual data
+    p.ind <- unique(sapply(X, ncol))
+    if (length(p.ind) != 1) {
+      variable.tmp <- sapply(X, function(xx) {
+        if (!is.null(colnames(xx))) TRUE else FALSE
+      })
+      if (!all(variable.tmp)) {
+        warning("Error: X matrices do not have the same number of variables. Provide variable names to the colnames of X matrix.")
+        return(NULL)
+      }
+    } else {
+      # --- check if there is only one X, default variable names as X_1, ..., X_p
+      X <- lapply(X, function(xx) {
+        if (is.null(colnames(xx))) {
+          colnames(xx) <- paste0("X_", 1:ncol(xx))
+        }
+        return(xx)
+      })
+    }
+    
+    # Remove duplicates and report: if duplicate columns of X
+    X <- lapply(seq_along(X), function(i) {
+      df <- X[[i]]
+      if (anyDuplicated(colnames(df))) {
+        message(paste("Removed duplicate columns from X matrix ", i))
+        df[, !duplicated(colnames(df)), drop = FALSE]
+      } else {
+        df
+      }
+    })
+    keep_variable_individual <- lapply(X, colnames)
+    if (!is.list(X) & !is.list(Y)) {
+      warning("Error: Input X and Y must be the list containing genotype matrics and all phenotype vectors!")
+      return(NULL)
+    } else {
+      if (length(X) == 1) {
+        yx_dict <- rep(1, length(Y))
+      } else if (length(X) == length(Y)) {
+        yx_dict <- 1:length(Y)
+      } else {
+        if (is.null(dict_YX)) {
+          warning("Error: Please provide the dict_YX since you have multiple Y but only few X!")
+          return(NULL)
+        } else {
+          # - dict for Y to X mapping
+          yx_dict <- rep(NA, length(Y))
+          for (i in 1:length(Y)) {
+            tmp <- unique(dict_YX[dict_YX[, 1] == i, 2])
+            if (length(tmp) == 0) {
+              warning(paste("Error: You don't provide matched X for outcome", i))
+              return(NULL)
+            } else if (length(tmp) != 1) {
+              warning(paste("Error: You provide different matched X for outcome", i))
+              return(NULL)
+            } else {
+              yx_dict[i] <- tmp
+            }
+          }
+          # - check if Y and X all mapped
+          if (max(yx_dict) > length(X)) {
+            warning("Error: You don't provide enough X matrices!")
+            return(NULL)
+          }
+        }
+      }
+    }
+    # keep_variable_individual <- lapply(yx_dict, function(i) keep_variable_individual[[i]] )
+    if (any(sapply(X, anyNA))) {
+      warning("Error: Input X must not contain missing values.")
+      return(NULL)
+    }
+    if (any(sapply(Y, anyNA))) {
+      pos <- which(sapply(Y, anyNA))
+      if (length(pos) == 1) {
+        Y_miss <- Y[[pos]]
+        samples_kept <- which(!is.na(Y_miss))
+        Y[[pos]] <- as.matrix(Y_miss[samples_kept])
+        # - check if only y for x
+        x_dict <- yx_dict[pos]
+        X_miss <- X[[x_dict]]
+        X_miss <- X_miss[samples_kept, ]
+        if (sum(yx_dict == x_dict) != 1) {
+          x_dict <- max(yx_dict) + 1
+          yx_dict[x_dict] <- x_dict
+        }
+        X[[x_dict]] <- X_miss
+      } else {
+        for (i in pos) {
+          Y_miss <- Y[[i]]
+          samples_kept <- which(!is.na(Y_miss))
+          Y[[i]] <- as.matrix(Y_miss[samples_kept])
+          # - check if only y for x
+          x_dict <- yx_dict[i]
+          X_miss <- X[[x_dict]]
+          X_miss <- X_miss[samples_kept, ]
+          if (sum(yx_dict == x_dict) != 1) {
+            x_dict <- max(yx_dict) + 1
+            yx_dict[x_dict] <- x_dict
+          }
+          X[[x_dict]] <- X_miss
+        }
+      }
+    }
+  } else {
+    yx_dict <- keep_variable_individual <- NULL
+  }
+  
+  # - check summary-level data
+  if ((!is.null(sumstat)) | (!is.null(effect_est) & !is.null(effect_se))) {
+    # --- check input of (effect_est, effect_se)
+    if ((!is.null(effect_est) & !is.null(effect_se))) {
+      if (!all(dim(effect_est) == dim(effect_se))) {
+        warning("Error: effect_est and effect_se should be the same dimension! Please check!")
+        return(NULL)
+      }
+      effect_est <- as.matrix(effect_est)
+      effect_se <- as.matrix(effect_se)
+      variables <- rownames(effect_est)
+      if (is.null(variables)) {
+        variables <- paste0("variant_", 1:nrow(effect_est))
+      } else {
+        # add - if duplciates occurs, only keep one
+        if (anyDuplicated(variables)) {
+          keep_idx <- !duplicated(variables)
+          variables <- variables[keep_idx]
+          effect_est <- effect_est[keep_idx, , drop = FALSE]
+          effect_se <- effect_se[keep_idx, , drop = FALSE]
+        }
+      }
+      sumstat <- list()
+      for (sum_iy in 1:ncol(effect_est)) {
+        sumstat_y <- data.frame(
+          "beta" = effect_est[, sum_iy],
+          "sebeta" = effect_se[, sum_iy],
+          "variant" = variables
+        )
+        if (!is.null(effect_n)) {
+          if (length(effect_n) == 1) {
+            sumstat_y$n <- effect_n
+          } else if (length(effect_n) == ncol(effect_est)) {
+            sumstat_y$n <- effect_n[sum_iy]
+          } else {
+            warning("effect_n does not match effect_est, ignoring sample size!")
+          }
+        }
+        sumstat[[sum_iy]] <- sumstat_y
+      }
+    }
+    
+    if (!is.null(sumstat)) {
+      if (is.data.frame(sumstat)) {
+        sumstat <- list(sumstat)
+      }
+      if (!is.list(sumstat)) {
+        warning("Error: Input sumstat must be the list containing summary level data for all outcomes!")
+        return(NULL)
+      }
+      sumstat <- lapply(sumstat, as.data.frame)
+      # --- check if variables names in summary data
+      variable.tmp <- sapply(sumstat, function(xx) {
+        if (("variant" %in% colnames(xx))) {
+          TRUE
+        } else {
+          FALSE
+        }
+      })
+      if (!all(variable.tmp)) {
+        warning(
+          "sumstat do not contain variables names. Assigning default names as 'variant_1, ..., variant_P'. ",
+          "Recommend to provide variables names to the column named 'variant'."
+        )
+        sumstat <- lapply(sumstat, function(xx) {
+          if (!("variant" %in% colnames(xx))) {
+            xx$variant <- paste0("variant_", 1:nrow(xx))
+          }
+          return(xx)
+        })
+      }
+    }
+    
+    # Remove NA for sumstat$variant columns - add on
+    sumstat <- lapply(seq_along(sumstat), function(i) {
+      xx <- sumstat[[i]]
+      if (anyNA(xx$variant)) {
+        warning(paste("Removed variant with NA from sumstat", i))
+        xx = as.data.frame(xx[!is.na(xx$variant), , drop = FALSE])
+      }
+      return(xx)
+    })
+    # Remove duplicates and report: if duplicate variant in summary statistics
+    sumstat <- lapply(seq_along(sumstat), function(i) {
+      xx <- sumstat[[i]]
+      if (anyDuplicated(xx$variant)) {
+        warning(paste("Removed duplicate variants from sumstat", i))
+        xx = as.data.frame(xx[!duplicated(xx$variant), , drop = FALSE])
+      }
+      return(xx)
+    })
+    
+    # --- check input of LD
+    M_updated <- M
+    min_abs_corr_updated <- min_abs_corr
+    jk_equiv_corr_updated <- 0.8
+    jk_equiv_loglik_updated <- 1
+    func_simplex_updated <- "LD_z2z"
+    
+    if (is.null(LD)) {
+      # if no LD input, set diagonal matrix to LD
+      warning(
+        "Providing the LD for summary statistics data is highly recommended. ",
+        "Without LD, only a single iteration will be performed under the assumption of one causal variable per outcome. ",
+        "Additionally, the purity of CoS cannot be evaluated!"
+      )
+      
+      LD <- 1
+      sumstatLD_dict <- rep(1, length(sumstat))
+      # change some algorithm parameters
+      M_updated <- 1 # one iteration
+      min_abs_corr_updated <- 0 # remove purity checking
+      jk_equiv_corr_updated <- 0
+      jk_equiv_loglik_updated <- 0.1
+      func_simplex_updated <- "only_z2z"
+      
+    } else {
+      
+      if (is.data.frame(LD)) LD <- as.matrix(LD)
+      if (is.matrix(LD)) LD <- list(LD)
+      # - check if NA in LD matrix
+      num_na <- sapply(LD, sum)
+      if (any(is.na(num_na))){
+        warning("Error: Input LD must not contain missing values (NA).")
+        return(NULL)
+      }
+      # Create sumstat-LD mapping ===
+      if (length(LD) == 1) {
+        sumstatLD_dict <- rep(1, length(sumstat))
+      } else if (length(LD) == length(sumstat)) {
+        sumstatLD_dict <- seq_along(sumstat)
+      } else {
+        if (is.null(dict_sumstatLD)) {
+          warning('Error: Please provide dict_sumstatLD: you have ', length(sumstat), 
+                  ' sumstats but only ', length(LD), ' LD matrices')
+          return(NULL)
+        } else {
+          # - dict for sumstat to LD mapping
+          sumstatLD_dict <- rep(NA, length(sumstat))
+          for (i in 1:length(sumstat)) {
+            tmp <- unique(dict_sumstatLD[dict_sumstatLD[, 1] == i, 2])
+            if (length(tmp) == 0) {
+              warning(paste("Error: You don't provide matched LD for sumstat", i))
+              return(NULL)
+            } else if (length(tmp) != 1) {
+              warning(paste("Error: You provide multiple matched LD for sumstat", i))
+              return(NULL)
+            } else {
+              sumstatLD_dict[i] <- tmp
+            }
+          }
+          if (max(sumstatLD_dict) > length(LD)) {
+            warning("Error: You don't provide enough LD matrices!")
+            return(NULL)
+          }
+        }
+      }
+      
+      # === Filter variants for each sumstat ===
+      for (i in seq_along(sumstat)) {
+        # Get sumstat variants (adjust field name based on your data structure)
+        sumstat_variants <- sumstat[[i]]$variant
+        n_total <- length(sumstat_variants)
+        # Get LD variants
+        ld_idx <- sumstatLD_dict[i]
+        current_ld <- LD[[ld_idx]]
+        ld_variants <- rownames(current_ld)
+        if (is.null(ld_variants)) {
+          if (ncol(current_ld) != n_total){
+            warning('Error: LD matrix ', ld_idx, ' has no rownames. Please ensure all LD matrices have variant names as rownames.')
+            return(NULL)
+          }
+        }
+        # Find common variants
+        common_variants <- intersect(sumstat_variants, ld_variants)
+        n_removed <- n_total - length(common_variants)
+        # Filter if needed
+        if (n_removed > 0) {
+          warning('Sumstat ', i, ': removing ', n_removed, ' out of ', n_total,
+                  ' variants since those variants are not in LD matrix ', ld_idx)
+          keep_idx <- match(common_variants, sumstat_variants)
+          if (length(keep_idx) == 0){
+            warning('Error: Sumstat data ', i, ' is empty after filtering. Returning NULL')
+            return(NULL)
+          }
+          # Filter all relevant fields - ADJUST THESE FIELD NAMES TO YOUR DATA
+          sumstat[[i]] <- sumstat[[i]][keep_idx, , drop = FALSE]
+        }
+      }
+    }
+    keep_variable_sumstat <- lapply(sumstat, function(xx) {
+      xx$variant
+    })
+    
+    # - checking sample size existency
+    n_exist <- sapply(sumstat, function(ss) {
+      "n" %in% colnames(ss)
+    })
+    if (!all(n_exist)) {
+      # The sample size (n) is not provided, so use unadjusted z-scores.
+      # The choice of n=2, yty=1 is mostly arbitrary except in that it
+      # ensures var(y) = yty/(n-1) = 1
+      p_no <- which(!n_exist)
+      warning(
+        "Providing the sample size (n), or even a rough estimate of n, ",
+        "is highly recommended. Without n, the implicit assumption is ",
+        "n is large (Inf) and the effect sizes are small (close to zero). ",
+        "Outcome ", paste(p_no, collapse = ", "), " in sumstat don't contain 'n'!"
+      )
+    }
+    
+    Z <- N_sumstat <- Var_y <- SeBhat <- vector(mode = "list", length = length(sumstat))
+    for (i.summstat in 1:length(sumstat)) {
+      summstat_tmp <- sumstat[[i.summstat]]
+      colVar <- colnames(summstat_tmp)
+      # - check p-value
+      if (!("z" %in% colVar)) {
+        if (!("beta" %in% colVar) || !("sebeta" %in% colVar)) {
+          warning("Error: Please provide either z or (bhat, sebhat)!")
+          return(NULL)
+        }
+        bhat <- summstat_tmp[, "beta"]
+        sebhat <- summstat_tmp[, "sebeta"]
+        if (anyNA(bhat) || anyNA(sebhat)) {
+          warning("Error: bhat, sebhat cannot have missing values")
+          return(NULL)
+        }
+        if (any(sebhat <= 0)) {
+          warning("Error: sebhat cannot have zero or negative elements")
+          return(NULL)
+        }
+        z <- bhat / sebhat
+        SeBhat[[i.summstat]] <- sebhat
+      } else {
+        z <- summstat_tmp[, "z"]
+      }
+      if (anyNA(z)) {
+        warning(paste("summary statistic dataset", i.summstat, "contains NA values that are replaced with 0"))
+        z[is.na(z)] <- 0
+      }
+      
+      # - check N
+      if (!("n" %in% colVar)) {
+        z <- z
+      } else {
+        n <- summstat_tmp[, "n"]
+        n <- as.numeric(na.omit(n)) # add hoc
+        if (!all(n > 1)) {
+          warning("Error: Sample size N must be greater than 1!")
+          return(NULL)
+        }
+        n <- median(n)
+        N_sumstat[[i.summstat]] <- n
+        
+        # When n is provided, compute the adjusted z-scores.
+        z <- z * sqrt((n - 1) / (z^2 + n - 2))
+        if ("var_y" %in% colVar) {
+          var_y <- unique(summstat_tmp[, "var_y"])
+          # --- need to check
+          if (length(var_y) != 1) {
+            var_y <- 1
+          }
+          if (var_y <= 0) {
+            warning("Error: var_y cannot have zero or negative elements")
+            return(NULL)
+          }
+          Var_y[[i.summstat]] <- var_y
+        }
+      }
+      Z[[i.summstat]] <- z
+    }
+  } else {
+    Z <- N_sumstat <- Var_y <- SeBhat <- sumstatLD_dict <- keep_variable_sumstat <- NULL
+    M_updated <- M
+    min_abs_corr_updated <- min_abs_corr
+    jk_equiv_corr_updated <- 0.8
+    jk_equiv_loglik_updated <- 1
+    func_simplex_updated <- "LD_z2z"
+  }
+  
+  return(list(
+    X = X,
+    Y = Y,
+    yx_dict = yx_dict,
+    keep_variable_individual = keep_variable_individual,
+    sumstat = sumstat,
+    LD = LD,
+    sumstatLD_dict = sumstatLD_dict,
+    keep_variable_sumstat = keep_variable_sumstat,
+    Z = Z,
+    N_sumstat = N_sumstat,
+    Var_y = Var_y,
+    SeBhat = SeBhat,
+    M = M_updated,
+    min_abs_corr = min_abs_corr_updated,
+    jk_equiv_corr = jk_equiv_corr_updated,
+    jk_equiv_loglik = jk_equiv_loglik_updated,
+    func_simplex = func_simplex_updated
+  ))
+}
+
